/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////

#ifndef DbAssoc2dConstraintGroup_INCLUDED_
#define DbAssoc2dConstraintGroup_INCLUDED_ /* {Secret} */

#include "DbConstraintsExports.h"
#include "DbAssocAction.h"
#include "OdConstraintGroupNode.h"
#include "OdConstrainedGeometry.h"
#include "OdGeomConstraint.h"
#include "OdExplicitConstr.h"

class OdDbAssocGeomDependency;
class OdAutoConstrainEvaluationCallback;

#include "TD_PackPush.h"

/** Description:
  Library: Source *code* provided.
  {group:Contraint_Classes} 
*/
class DBCONSTRAINTS_EXPORT OdDbAssoc2dConstraintGroup : public OdDbAssocAction
{
public:
  ODRX_DECLARE_MEMBERS(OdDbAssoc2dConstraintGroup);

  enum SolutionStatus
  {
    kWellDefined = 0,
    kUnderConstrained,
    kOverConstrained,
    kInconsistent,
    kNotEvaluated,
    kNotAvailable,
    kRejectedByClient,
  };

  OdDbAssoc2dConstraintGroup();

  OdResult setWorkPlane(const OdGePlane plane);
  const OdGePlane getWorkPlane() const;

#if 0
  OdResult transformActionBy(const OdGeMatrix3d& transform); 

  int getDOF() const;

  SolutionStatus solutionStatus(bool bAlsoCheckForConstraints = true) const;

  SolutionStatus geometryStatus(const OdConstrainedGeometry* pConsGeom) const;

  SolutionStatus constraintStatus(const OdGeomConstraint* pConstraint) const;
#endif
  OdResult addConstrainedGeometry(const OdDbFullSubentPath& subentPath, OdConstrainedGeometry*& pConsGeom);

  OdResult deleteConstrainedGeometry(const OdDbObjectId& geomDependencyId);


  OdResult getConstrainedGeometry(const OdDbFullSubentPath& fullSubentPath,
                                  OdConstrainedGeometry*&   pConsGeom,
                                  bool bCreateArcLineMid = false);

  OdResult getConstrainedGeometry(const OdDbAssocGeomDependency* pGeomDependency,
                                  OdConstrainedGeometry*&   pConsGeom,
                                  OdConstrainedImplicitPoint::ImplicitPointType* pPtType = 0,
                                  int defPtIndex = -1,
                                  bool bCreateArcLineMid = false);

  OdResult getConstrainedGeometries(OdArray<OdConstrainedGeometry*>& aConstGeom) const;


  OdResult addGeometricalConstraint(OdGeomConstraint::GeomConstraintType type, 
                                    const OdDbFullSubentPathArray& aPaths,
                                    OdGeomConstraint** ppNewConstraint  = NULL);

  OdResult addGeometricalConstraint(OdGeomConstraint::GeomConstraintType type,
                                    const OdArray<OdConstrainedGeometry*>& apConsGeoms,
                                    OdGeomConstraint** ppNewConstraint  = NULL);
  OdResult addDistanceConstraint(OdConstrainedGeometry* pConsGeom1,
                                 OdConstrainedGeometry* pConsGeom2,
                                 OdDistanceConstraint::DirectionType directionType,
                                 OdDbObjectId valueDependencyId,
                                 OdDbObjectId dimDependencyId, // May be null
                                 const OdGeVector3d* pFixedDirection = NULL,
                                 const OdConstrainedLine* pDirectionLine = NULL,
                                 OdDistanceConstraint** ppNewDisConstraint = NULL);

  OdResult addAngleConstraint(OdConstrainedLine* pConsLine1,
                              OdConstrainedLine* pConsLine2,
                              OdAngleConstraint::SectorType sectorType,
                              OdDbObjectId valueDependencyId,
                              OdDbObjectId dimDependencyId, // May be null
                              OdAngleConstraint** ppNewAngConstraint = NULL);

  OdResult add3PointAngleConstraint(OdConstrainedPoint* pConsPoint1,
                                    OdConstrainedPoint* pConsPoint2,
                                    OdConstrainedPoint* pConsPoint3,
                                    OdAngleConstraint::SectorType sectorType,
                                    OdDbObjectId valueDependencyId,
                                    OdDbObjectId dimDependencyId, // May be null
                                    Od3PointAngleConstraint** ppNewAngConstraint = NULL);

  OdResult addRadiusDiameterConstraint(OdConstrainedGeometry* pConsGeom,
                                       OdRadiusDiameterConstraint::RadiusDiameterConstrType type,
                                       OdDbObjectId valueDependencyId,
                                       OdDbObjectId dimDependencyId, // May be null
                                       OdRadiusDiameterConstraint** ppNewRadDiaConstraint = NULL);

  OdResult deleteConstraint(OdGeomConstraint* pGeomConst);

  OdResult getConstraints(OdArray<OdGeomConstraint*>& apConstraints) const;
#if 0
  OdResult autoConstrain(const OdDbFullSubentPathArray& aPaths,
                         const OdGeTol& tol,
                         OdAutoConstrainEvaluationCallback* pCallback);
#endif
  OdResult getAllConnectedGeomDependencies(const OdDbObjectIdArray& srcGeomDependencyIds,
                                           OdDbObjectIdArray& allConnectedGeomDependencyIds) const;

  OdConstraintGroupNode* getGroupNodePtr(const OdConstraintGroupNodeId nodeId) const;
#if 0
  void regenDimensionSystem();

  OdResult geometryMirrored(OdDbAssocGeomDependency* pGeomDependency);
  
  static void addGlobalCallback(class OdDbAssoc2dConstraintCallback* pCallback);

  static void removeGlobalCallback(class OdDbAssoc2dConstraintCallback* pCallback);

  static class OdDbAssoc2dConstraintCallback* globalCallback();
#endif

  //remove in full version
}; // class OdDbAssoc2dConstraintGroup

typedef OdSmartPtr<OdDbAssoc2dConstraintGroup> OdDbAssoc2dConstraintGroupPtr;

/** Description:
  Library: Source *code* provided.
  {group:Contraint_Classes} 
*/
class OdDbAssoc2dConstraintCallback
{
public:
    OdDbAssoc2dConstraintCallback() {}

    virtual ~OdDbAssoc2dConstraintCallback() {}

    virtual bool canBeRelaxed(const class OdExplicitConstraint* /*pConstraint*/)
    {
        return false;
    }

    virtual void constraintDeactivated(const OdExplicitConstraint * /*pConstraint*/,
                                       bool /* bDeactivated */ = true) {}
};

/** Description:
  Library: Source *code* provided.
  {group:Contraint_Classes} 
*/
class DBCONSTRAINTS_EXPORT OdAutoConstrainEvaluationCallback
{
    friend class OdDbImpAssoc2dConstraintGroup;
    friend class OdAutoConstraintEvalManager;
public:
  virtual void getConstraintPriorityOverride(OdGeomConstraint::GeomConstraintType /*type*/, 
        const OdArray<const OdConstrainedGeometry*>& /*geometries*/, int& /*priority*/) const
  {
      return; //No override.
  }

  virtual void getAutoConstrainPriority(OdArray<OdGeomConstraint::GeomConstraintType>& constraintList) const
  {
      constraintList.setLogicalLength(0);
      for(int type = (int)OdGeomConstraint::kHorizontal;
          type < (int)OdGeomConstraint::kEqualRadius; ++type)
      {
          constraintList.append((OdGeomConstraint::GeomConstraintType)type);
      }
  }

  virtual bool isEvaluationCancelled() { return false; }

  int getTotalConstraints(OdArray<OdGeomConstraint*>* pConstraints) const;

private:  
  int getAutoConstraintMask() const;

  void validate(void* pData);

  bool isValidConstraintType(OdGeomConstraint::GeomConstraintType type,
        const OdGeomConstraint* pConstraint) const;

  void constraintAdded(OdGeomConstraint* pNewCons);

private:
    OdArray<OdGeomConstraint*> mConstraints;
};

#include "TD_PackPop.h"

#endif

