/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////

#ifndef DbAssocActionBody_INCLUDED_
#define DbAssocActionBody_INCLUDED_ /* {Secret} */

#include "DbConstraintsExports.h"
#include "DbObject.h"
#include "DbAssocAction.h"
#include "DbEntity.h"

#include "TD_PackPush.h"

class DBCONSTRAINTS_EXPORT OdDbAssocActionBody : public OdDbObject
{
public: 
  ODRX_DECLARE_MEMBERS(OdDbAssocActionBody);

  OdDbAssocActionBody();
  //explicit AcDbAssocActionBody(AcDbAssocCreateImpObject createImpObject = kAcDbAssocCreateImpObject);
  virtual ~OdDbAssocActionBody();

  OdDbObjectId parentAction() const { return ownerId(); }
  OdDbAssocStatus status() const;
  OdResult setStatus(OdDbAssocStatus newStatus, 
                      bool           notifyOwningNetwork = true,
                      bool           setInOwnedActions   = false);

  OdDbObjectId owningNetwork() const;

  OdResult getDependencies(bool readDependenciesWanted, 
                           bool writeDependenciesWanted,
                           OdDbObjectIdArray& dependencyIds) const; 

  OdResult addDependency(const OdDbObjectId& dependencyId, 
                         bool  setThisActionAsOwningAction = true); 

  OdResult removeDependency(const OdDbObjectId& dependencyId, 
                                  bool          alsoEraseIt);

  OdResult removeAllDependencies(bool alsoEraseThem);

  OdResult evaluateDependencies();

  bool isActionEvaluationInProgress() const;

  OdDbAssocEvaluationCallback* currentEvaluationCallback() const;
public: 
  virtual void evaluateOverride() = 0;
  OdResult getDependenciesOverride(bool readDependenciesWanted, 
                                   bool writeDependenciesWanted,
                                   OdDbObjectIdArray& dependencyIds) const
  { return eNotImplemented; }

  OdResult getDependentObjectsOverride(bool readDependenciesWanted,
                                       bool writeDependenciesWanted,
                                       OdDbObjectIdArray& objectIds) const
  { return eNotImplemented; }

  virtual OdResult removeAllDependenciesOverride(bool alsoEraseThem)
  { return eNotImplemented; }

  virtual OdResult isOwnedDependencyOverride(const OdDbAssocDependency* pDependency, 
                                             bool& isOwnedDependency) const
  { return eNotImplemented; }

  virtual OdResult isExternalDependencyOverride(const OdDbAssocDependency* pDependency, 
                                                      bool& isExternalDependency) const
  { return eNotImplemented; }

  virtual OdResult hasDependencyCachedValueOverride(const OdDbAssocDependency* pDependency,
                                                             bool& hasDepCachedValue) const
  { return eNotImplemented; }

  virtual OdResult isRelevantDependencyChangeOverride(const OdDbAssocDependency* pDependency,
                                                      bool& isRelevantDepChange) const
  { return eNotImplemented; }

  virtual OdResult areDependenciesOnTheSameThingOverride(const OdDbAssocDependency* pDependency1,
                                                         const OdDbAssocDependency* pDependency2,
                                                         bool& areDependentOnSameThing) const
  { return eNotImplemented; }

  virtual OdResult areDependenciesEqualOverride(const OdDbAssocDependency* pDependency1,
                                                const OdDbAssocDependency* pDependency2,
                                                bool& areEqual) const
  { return eNotImplemented; }

  virtual OdResult evaluateDependencyOverride(OdDbAssocDependency* pDependency)
  { return eNotImplemented; }

  virtual OdResult dependentObjectClonedOverride(const OdDbAssocDependency* pDependency, 
                                                 const OdDbObject*          pDbObj, 
                                                 const OdDbObject*          pNewObj)
  { return eNotImplemented; }

  virtual OdResult addMoreObjectsToDeepCloneOverride(OdDbIdMapping& idMap, 
                                                     OdDbObjectIdArray& additionalObjectsToClone) const
  { return eNotImplemented; }

  virtual OdResult postProcessAfterDeepCloneOverride(OdDbIdMapping& idMap) 
  { return eNotImplemented; }

  virtual OdResult postProcessAfterDeepCloneCancelOverride(OdDbIdMapping& idMap)
  { return eNotImplemented; }

  virtual OdResult ownedDependencyStatusChangedOverride(OdDbAssocDependency* pOwnedDependency, 
                                                        OdDbAssocStatus      previousStatus)
  { return eNotImplemented; }

  virtual OdResult isEqualToOverride(const OdDbAssocAction* pOtherAction, bool& isEqual) const
  { return eNotImplemented; }

  virtual OdResult evaluationPriorityOverride(OdDbAssocEvaluationPriority& priority) const
  { return eNotImplemented; }

  virtual OdResult getDependentActionsToEvaluateOverride(OdDbActionsToEvaluateCallback* pActionsToEvaluateCallback) const
  { return eNotImplemented; }

  virtual OdResult transformActionByOverride(const OdGeMatrix3d&)
  { return eNotImplemented; }

  virtual OdResult dragStatusOverride(const OdDb::DragStat status)
  { return eNotImplemented; }
protected:
};

typedef OdSmartPtr<OdDbAssocActionBody> OdDbAssocActionBodyPtr;

#include "TD_PackPop.h"
#endif

