/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////

#ifndef DbAssocGeomDependency_INCLUDED_
#define DbAssocGeomDependency_INCLUDED_ /* {Secret} */

#include "DbConstraintsExports.h"
#include "DbAssocDependency.h"
#include "DbAssocPersSubentId.h"
#include "Ge/GePoint3dArray.h"

class OdGeCurve3d;
class OdGeSurface;

#include "TD_PackPush.h"

class DBCONSTRAINTS_EXPORT OdDbAssocGeomDependency : public OdDbAssocDependency
{
public: 
  ODRX_DECLARE_MEMBERS(OdDbAssocGeomDependency);

  OdDbAssocGeomDependency(bool createImp = true);

  OdDbAssocPersSubentIdPtr persistentSubentId() const;

  OdResult getTransientSubentIds(OdArray<OdDbSubentId>& transientSubentIds) const;

  OdResult setSubentity(const OdDbSubentId& transientSubentId);

  OdDb::SubentType subentType() const;

  int transientSubentCount() const;

  bool isCachingSubentityGeometry() const;

  void setCachingSubentityGeometry(bool bCaching);

  OdResult getVertexSubentityGeometry(OdGePoint3dArray& vertexPositions) const;

  OdResult getEdgeSubentityGeometry(OdArray<OdGeCurve3d*>& edgeCurves) const;

  OdResult getFaceSubentityGeometry(OdArray<OdGeSurface*>& faceSurfaces) const;

  OdResult setVertexSubentityGeometry(const OdGePoint3dArray& newVertexPositions);

  OdResult setEdgeSubentityGeometry(const OdArray<const OdGeCurve3d*>& newEdgeCurves);

  OdResult setFaceSubentityGeometry(const OdArray<const OdGeSurface*>& newFaceSurfaces);

  OdResult dependentOnObjectMirrored();

  static OdResult redirectToAnotherSubentity(const OdDbObjectId& oldObjectId,
                                             const OdDbSubentId& oldSubentId,
                                             const OdDbObjectId& newObjectId,
                                             const OdDbSubentId& newSubentId);
}; // class OdDbAssocGeomDependency

typedef OdSmartPtr<OdDbAssocGeomDependency> OdDbAssocGeomDependencyPtr;

#include "TD_PackPop.h"

#endif
