/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////

#ifndef DbAssocGlobal_INCLUDED_
#define DbAssocGlobal_INCLUDED_ /* {Secret} */

#include "DbObject.h"
#include "Ge/GeTol.h"

class OdDbAssocAction;
class OdDbAssocActionBody;
class OdDbAssocNetwork;
class OdDbAssocDependency;
class OdDbAssocDependencyBody;
class OdDbAssocGeomDependency;
class OdDbAssoc2dConstraintGroup;
class OdDbAssocPersSubentId;
class OdDbAssocSimplePersSubentId;
class OdDbAssocSingleEdgePersSubentId;
class OdDbAssocPersSubentIdPE;
class OdDbAssocVariable;
class OdDbAssocValueDependency;
class OdDbAssocValueProviderPE;
class OdDbEvalContext;

#include "TD_PackPush.h"

namespace Constraint
{
enum ErrorStatus
{
        kSuccess,
        kOutOfRange,
        kInvalidInput
};
};

enum OdDbAssocStatus
{
  kIsUpToDateAssocStatus = 0,
  kChangedDirectlyAssocStatus,
  kChangedTransitivelyAssocStatus,
  kChangedNoDifferenceAssocStatus, 
  kFailedToEvaluateAssocStatus,
  kErasedAssocStatus,
  kSuppressedAssocStatus,  
};

inline int evaluationRequestSeverityLevel(OdDbAssocStatus status)
{
    switch (status)
    {
    case kChangedDirectlyAssocStatus:
        return 3;
    case kChangedTransitivelyAssocStatus:
        return 2;
    case kChangedNoDifferenceAssocStatus:
        return 1;
    default:
        return 0; // Not a request to evaluate
    }
}

inline bool isEvaluationRequest(OdDbAssocStatus status)
{
    return evaluationRequestSeverityLevel(status) > 0;
}

inline bool isToBeSkipped(OdDbAssocStatus status)
{
    return status == kErasedAssocStatus || status == kSuppressedAssocStatus;
}

enum OdDbAssocEvaluationPriority
{
  kCannotBeEvaluatedAssocEvaluationPriority = -1000,
  kCannotDermineAssocEvaluationPriority = 0,
  kCanBeEvaluatedAssocEvaluationPriority = 1000,
};

enum OdDbAssocEvaluationMode
{
    kModifyObjectsAssocEvaluationMode,
    kModifyActionAssocEvaluationMode,
};

enum OdDbAssocDraggingState
{
    kNotDraggingAssocDraggingState,
    kFirstSampleAssocDraggingState,
    kIntermediateSampleAssocDraggingState,
    kLastSampleAssocDraggingState,
};

enum OdDbAssocTransformationType
{
    kNotSpecified,
    kStretch,
    kRotate,
    kMove,
};

class DBCONSTRAINTS_EXPORT OdDbAssocEvaluationCallback
{
public:
    OdDbAssocEvaluationCallback() {}
    virtual ~OdDbAssocEvaluationCallback() {}
    virtual OdDbAssocEvaluationMode evaluationMode() const { return kModifyObjectsAssocEvaluationMode; }
    virtual void beginActionEvaluation(OdDbAssocAction* pAction) = 0;
    virtual void endActionEvaluation(OdDbAssocAction* pAction) = 0;
    virtual void setActionEvaluationErrorStatus(OdDbAssocAction*    pAction,
                                                Constraint::ErrorStatus   errorStatus,
                                                const OdDbObjectId& objectId   = OdDbObjectId::kNull,
                                                OdDbObject*         pObject    = NULL,
                                                void*               pErrorInfo = NULL) = 0;

    virtual void 
    beginActionEvaluationUsingObject(OdDbAssocAction*    pAction, 
                                     const OdDbObjectId& objectId, 
                                     bool                objectIsGoingToBeUsed,
                                     bool                objectIsGoingToBeModified,
                                     OdDbObject*&        pSubstituteObject) = 0;

    virtual void endActionEvaluationUsingObject(OdDbAssocAction*    pAction, 
                                                const OdDbObjectId& objectId, 
                                                OdDbObject*         pObject) = 0;

    virtual OdDbAssocDraggingState draggingState() const { return kNotDraggingAssocDraggingState; }

    virtual bool cancelActionEvaluation() = 0;

    virtual OdDbEvalContext* getAdditionalData() const { return NULL; }

    virtual OdDbAssocTransformationType getTransformationType() const { return kNotSpecified; }
};

inline bool isDraggingProvidingSubstituteObjects(const OdDbAssocEvaluationCallback* pEvaluationCallback)
{
    if (pEvaluationCallback == NULL)
        return false;

    const OdDbAssocDraggingState draggingState = pEvaluationCallback->draggingState();

    return draggingState == kFirstSampleAssocDraggingState || 
           draggingState == kIntermediateSampleAssocDraggingState;
}

class DBCONSTRAINTS_EXPORT OdDbActionsToEvaluateCallback
{
public:
    OdDbActionsToEvaluateCallback() {}

    virtual ~OdDbActionsToEvaluateCallback() {}

    virtual void needsToEvaluate(const OdDbObjectId& objectId, 
                                 OdDbAssocStatus     newStatus,
                                 bool                ownedActionsAlso = true) = 0;

};

enum OdDbAssocCreateImpObject 
{
    kAcDbAssocCreateImpObject = 0,
    kAcDbAssocDoNotCreateImpObject = 1,
};

#include "TD_PackPop.h"

#endif

