/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////

#ifndef DbAssocPersSubentId_INCLUDED_
#define DbAssocPersSubentId_INCLUDED_ /* {Secret} */

#include "DbConstraintsExports.h"
#include "DbSubentId.h"
#include "SmartPtr.h"

class OdDbEntity;
class OdDbDwgFiler;
class OdDbDxfFiler;
class OdDbAuditInfo;

class OdDbAssocPersSubentId;
typedef OdSmartPtr<OdDbAssocPersSubentId> OdDbAssocPersSubentIdPtr;

class OdDbAssocSimplePersSubentId;
typedef OdSmartPtr<OdDbAssocSimplePersSubentId> OdDbAssocSimplePersSubentIdPtr;

class OdDbAssocSingleEdgePersSubentId;
typedef OdSmartPtr<OdDbAssocSingleEdgePersSubentId> OdDbAssocSingleEdgePersSubentIdPtr;

#include "TD_PackPush.h"

class DBCONSTRAINTS_EXPORT OdDbAssocPersSubentId : public OdRxObject
{
public:
	ODRX_DECLARE_MEMBERS(OdDbAssocPersSubentId);
  OdDbAssocPersSubentId() {}
  virtual ~OdDbAssocPersSubentId() {}

  virtual OdResult getTransientSubentIds(const OdDbEntity*      pEntity, 
                                         OdArray<OdDbSubentId>& subents) const;

  virtual OdDb::SubentType subentType(const OdDbEntity* pEntity) const = 0;

  virtual int transientSubentCount(const OdDbEntity* pEntity) const;

  virtual bool isNull() const = 0;

  virtual bool isEqualTo(const OdDbEntity* pEntity, const OdDbAssocPersSubentId* pOther) const
  {
    return this == pOther || pEntity != NULL && pOther != NULL && isA() == pOther->isA();
  }

  virtual OdResult mirror(const OdDbEntity* pMirroredEntity);

  static OdResult createObjectAndDwgInFields(OdDbDwgFiler* pFiler, OdDbAssocPersSubentIdPtr& pCreatedPersSubentId);
  static OdResult createObjectAndDxfInFields(OdDbDxfFiler* pFiler, OdDbAssocPersSubentIdPtr& pCreatedPersSubentId);

  virtual OdResult dwgOutFields(OdDbDwgFiler* pFiler) const;
  virtual OdResult dwgInFields(OdDbDwgFiler* pFiler);

  virtual void dxfOutFields(OdDbDxfFiler* pFiler) const;
  virtual OdResult dxfInFields(OdDbDxfFiler* pFiler);

  virtual void audit(OdDbAuditInfo* /*pAuditInfo*/) {}
};

//----------------------------------------------------------------------------

class DBCONSTRAINTS_EXPORT OdDbAssocSingleEdgePersSubentId : public OdDbAssocPersSubentId
{
public:
	ODRX_DECLARE_MEMBERS(OdDbAssocSingleEdgePersSubentId);

  OdDbAssocSingleEdgePersSubentId();
  virtual int transientSubentCount(const OdDbEntity* /*pEntity*/) const { return 1; }
  virtual OdDb::SubentType subentType(const OdDbEntity* /*pEntity*/) const { return OdDb::kEdgeSubentType; }
  virtual bool isNull() const { return false; }
  virtual bool isEqualTo(const OdDbEntity* pEntity, const OdDbAssocPersSubentId* pOther) const
  {
    return OdDbAssocPersSubentId::isEqualTo(pEntity, pOther);
  }

  virtual OdResult dwgOutFields(OdDbDwgFiler* pFiler) const;
  virtual OdResult dwgInFields(OdDbDwgFiler* pFiler);
  virtual void dxfOutFields(OdDbDxfFiler* pFiler) const;
  virtual OdResult dxfInFields(OdDbDxfFiler* pFiler);
protected:
  bool m_Unk_Bool_1;
  int m_Unk_UInt16_1;  
};

//----------------------------------------------------------------------------
class DBCONSTRAINTS_EXPORT OdDbAssocSimplePersSubentId : public OdDbAssocPersSubentId
{
public:
	ODRX_DECLARE_MEMBERS(OdDbAssocSimplePersSubentId);

  OdDbAssocSimplePersSubentId();
  void init(const OdDbSubentId& subentId);

  virtual int transientSubentCount(const OdDbEntity* /*pEntity*/) const { return !isNull() ? 1 : 0; }

  virtual OdDb::SubentType subentType(const OdDbEntity* /*pEntity*/) const { return m_SubentId.type(); }

  virtual OdDbSubentId subentId(const OdDbEntity* /*pEntity*/) const { return m_SubentId; }

  virtual bool isNull() const
  {
    return m_SubentId.type() == OdDb::kNullSubentType;
  }

  virtual bool isEqualTo(const OdDbEntity* pEntity, const OdDbAssocPersSubentId* pOther) const
  {
    if (!OdDbAssocPersSubentId::isEqualTo(pEntity, pOther))
      return false;

    OdDbAssocSimplePersSubentIdPtr pOtherSimple = pOther;
    return m_SubentId == pOtherSimple->m_SubentId;
  }

  virtual OdResult dwgOutFields(OdDbDwgFiler* pFiler) const;

  virtual OdResult dwgInFields(OdDbDwgFiler* pFiler);

  virtual void dxfOutFields(OdDbDxfFiler* pFiler) const;

  virtual OdResult dxfInFields(OdDbDxfFiler* pFiler);

  virtual void audit(OdDbAuditInfo* pAuditInfo);

protected:
  bool m_Unk_Bool_1;
private:
  OdDbSubentId m_SubentId;
};

#include "TD_PackPop.h"

#endif


