/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////

#ifndef DbAssocVariable_INCLUDED_
#define DbAssocVariable_INCLUDED_ /* {Secret} */

#include "DbConstraintsExports.h"

#include "DbAssocValueProviderPE.h"
#include "DbAssocAction.h"

#include "OdString.h"
#include "BoolArray.h"
#include "DoubleArray.h"
#include "Int16Array.h"

#include "DbEvalVariant.h"

#include "TD_PackPush.h"


class DBCONSTRAINTS_EXPORT OdDbAssocVariable : public OdDbAssocAction
{
public:     
  ODRX_DECLARE_MEMBERS(OdDbAssocVariable);

  OdDbAssocVariable();

  const OdString name() const;
  const OdString expression() const;
  const OdDbEvalVariantPtr value() const;
  const OdString description() const;

  OdResult setName(const OdString newName, bool updateReferencingExpressions);
  OdDbObjectId findObjectByName(const OdString  objectName, 
                                const OdRxClass* pObjectClass) const;
  OdResult validateNameAndExpression(const OdString nameToValidate, 
                                              const OdString expressionToValidate,
                                              OdString       errorMessage) const;

  OdResult setExpression(const OdString newExpression, 
                        const OdString evaluatorId, 
                        bool checkForCyclicalDependencies,
                        bool updateDependenciesOnReferencedSymbols,
                        OdString errorMessage = OdString::kEmpty,
                        bool silentMode = false);
  const OdString evaluatorId() const;
  OdResult setEvaluatorId(const OdString evalId);

  OdResult setValue(const OdDbEvalVariant& newValue);

  OdResult setDescription(const OdString newDescription);

  bool     isMergeable          () const;
  bool     mustMerge            () const;
  OdString mergeableVariableName() const;
  void setIsMergeable(bool isMerg, bool mustMerg = false, const OdString mergeableVariableName = OdString());

  OdResult evaluateExpression(OdDbEvalVariant& evaluatedExpressionValue,
                                       OdString        errorMessage = OdString()) const;

  OdResult evaluateExpression(OdDbObjectIdArray&        objectIds,    // in/out argument
                                       OdArray<OdDbEvalVariant>& objectValues, // in/out argument
                                       OdDbEvalVariant&          evaluatedExpressionValue,
                                       OdString                 errorMessage = OdString()) const;

  static void addGlobalCallback(class OdDbAssocVariableCallback* pCallback);

  static void removeGlobalCallback(class OdDbAssocVariableCallback* pCallback);

  static class OdDbAssocVariableCallback* globalCallback();
};


typedef OdSmartPtr<OdDbAssocVariable> OdDbAssocVariablePtr;

class OdDbAssocVariableCallback
{
public:
    OdDbAssocVariableCallback() {}
    virtual ~OdDbAssocVariableCallback() {}

    virtual bool canBeErased(const OdDbAssocVariable* pVariable) = 0;

    virtual OdResult validateNameAndExpression(const OdDbAssocVariable* pVariable,
                                              const OdString nameToValidate, 
                                              const OdString expressionToValidate,
                                              OdString errorMessage) = 0;
};


class OdDbAssocVariableValueProviderPE : public OdDbAssocValueProviderPE
{
public:
	ODRX_DECLARE_MEMBERS(OdDbAssocVariableValueProviderPE);
#if 0
  virtual OdResult getValue(const OdDbObject* pObject, 
                           const OdString   valueName, 
                           OdDbEvalVariant&  value);
#endif
};  

typedef OdSmartPtr<OdDbAssocVariableValueProviderPE> OdDbAssocVariableValueProviderPEPtr;

#include "TD_PackPop.h"

#endif
