/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////

#ifndef OdConstrainedGeometry_INCLUDED
#define OdConstrainedGeometry_INCLUDED

#include "OdaCommon.h"

#include "OdGeomConstraint.h"
#include "OdConstraintGroupNode.h"
#include "DbAssocGeomDependency.h"
#include "DbSubentId.h"
#include "Ge/GeLineSeg3d.h"
#include "Ge/GeRay3d.h"
#include "Ge/GeCircArc3d.h"
#include "Ge/GeEllipArc3d.h"
#include "Ge/GeNurbCurve3d.h"
#include "Ge/GeKnotVector.h"
#include "Ge/GePoint3dArray.h"

#include "TD_PackPush.h"


// Forward class declarations.
//
class OdConstrainedRigidSet;

class DBCONSTRAINTS_EXPORT OdConstrainedGeometry :  public OdConstraintGroupNode
{
public:
  ODRX_DECLARE_MEMBERS(OdConstrainedGeometry);

  OdResult init(const OdDbObjectId geomDependencyId);

  virtual bool isReadOnly() const;

  OdDbObjectId geomDependencyId() const;

  OdConstrainedRigidSet* getOwningRigidSet() const;

  OdResult getConnectedConstraints(OdArray<OdGeomConstraint*>& apConstraints) const;

  OdResult getCommonConstraints(OdConstrainedGeometry* pOtherConsGeom,
                                OdArray<OdGeomConstraint*>& apConstraints) const;

  OdResult getConnectedGeometries(OdArray<OdConstrainedGeometry*>& apConsGeoms) const;

  OdResult getFullSubentPaths(OdDbFullSubentPathArray& subentPaths) const;

  virtual bool operator==(const OdConstrainedGeometry& geom) const;
  virtual bool operator!=(const OdConstrainedGeometry& geom) const;
  OdResult setDataFromGeomDependency(const OdDbObjectId geomDependencyId,
                                     const OdGeMatrix3d& matrFromWrd);

  bool isModified() const;
  void DoPostEvaluateJob();
protected:
  OdConstrainedGeometry();
};


class DBCONSTRAINTS_EXPORT OdConstrainedPoint: public OdConstrainedGeometry 
{ 
public:
  ODRX_DECLARE_MEMBERS(OdConstrainedPoint);

  OdConstrainedPoint(bool bCreateImp = true);

  OdGePoint3d point() const;
  OdResult setPoint(const OdGePoint3d& val);

  const OdGePoint3d getOriginalPoint() const;
}; 

class DBCONSTRAINTS_EXPORT OdConstrainedImplicitPoint: public OdConstrainedPoint 
{
public:
  ODRX_DECLARE_MEMBERS(OdConstrainedImplicitPoint);

  enum ImplicitPointType 
  {
    kStartImplicit = 0,
    kEndImplicit,
    kMidImplicit,
    kCenterImplicit,
    kDefineImplicit
  };

  OdConstrainedImplicitPoint();
  OdResult init(OdConstraintGroupNodeId constrCurvId, ImplicitPointType ptype, int index = -1);

  ImplicitPointType       pointType()  const;
  OdInt32                 pointIndex() const;
  OdConstraintGroupNodeId constrainedCurveId() const;
};


class DBCONSTRAINTS_EXPORT OdConstrainedCurve :  public OdConstrainedGeometry
{
public:
  ODRX_DECLARE_MEMBERS(OdConstrainedCurve);

  virtual bool isBounded() const = 0;

  virtual void getConstrainedImplicitPoints(OdArray<OdConstrainedImplicitPoint*>& apImplicitPoints) const;

  OdResult point(const OdConstrainedImplicitPoint* pImplicitPoint, OdGePoint3d& res) const;
  OdResult setPoint(const OdConstrainedImplicitPoint* pImplicitPoint, const OdGePoint3d& val);
  OdResult getOriginalPoint(const OdConstrainedImplicitPoint* pImplicitPoint, OdGePoint3d& res) const;
protected:
  OdConstrainedCurve();
};

class DBCONSTRAINTS_EXPORT OdConstrainedLine: public OdConstrainedCurve 
{ 
public:
  ODRX_DECLARE_MEMBERS(OdConstrainedLine);
  
  OdConstrainedLine(bool bCreateImp = true);

  OdGePoint3d pointOnLine() const;
  OdGeVector3d direction() const;

  virtual bool isBounded() const;

  OdResult setPointOnLine(const OdGePoint3d& val);
  OdResult setDirection(const OdGeVector3d& val);

  OdResult set(const OdGePoint3d pointOnLine, const OdGeVector3d dir);

  const OdGeLine3d getOdGeLine3d() const;
  const OdGeLine3d getOriginalOdGeLine3d() const;
}; 

class DBCONSTRAINTS_EXPORT OdConstrainedConstructionLine: public OdConstrainedLine 
{ 
public:
  ODRX_DECLARE_MEMBERS(OdConstrainedConstructionLine);
protected:
  OdConstrainedConstructionLine();
};

class DBCONSTRAINTS_EXPORT OdConstrained2PointsConstructionLine: public OdConstrainedConstructionLine 
{ 
public:
  ODRX_DECLARE_MEMBERS(OdConstrained2PointsConstructionLine);
  OdConstrained2PointsConstructionLine();
};

class DBCONSTRAINTS_EXPORT OdConstrainedDatumLine: public OdConstrainedConstructionLine 
{ 
public:
  ODRX_DECLARE_MEMBERS(OdConstrainedDatumLine);
  OdConstrainedDatumLine();
};

class DBCONSTRAINTS_EXPORT OdConstrainedBoundedLine: public OdConstrainedLine
{
public:
  ODRX_DECLARE_MEMBERS(OdConstrainedBoundedLine);

  OdConstrainedBoundedLine();

  OdResult init(const OdDbObjectId depId, bool bRay);
  bool isRay() const;
  OdGePoint3d startPoint() const;
  OdGePoint3d endPoint() const;
  OdGePoint3d midPoint() const;

  virtual bool isBounded() const;

  OdResult setStartPoint(const OdGePoint3d& val);
  OdResult setEndPoint(const OdGePoint3d& val);

  const OdGeLineSeg3d getOdGeLineSeg3d() const;
  const OdGeLineSeg3d getOriginalOdGeLineSeg3d() const;

  OdResult set(const OdGePoint3d startPt, const OdGePoint3d endPt);

  const OdGeRay3d getOdGeRay3d() const;
  const OdGeRay3d getOriginalOdGeRay3d() const;
private:
  OdGeLineSeg3d m_originLine;
  OdGeLineSeg3d m_line;

  OdGeRay3d m_originRay;
  OdGeRay3d m_ray;

  bool m_bRay;
};

class DBCONSTRAINTS_EXPORT OdConstrainedCircle: public OdConstrainedCurve 
{ 
public:
  ODRX_DECLARE_MEMBERS(OdConstrainedCircle);
  OdConstrainedCircle(bool bCreateImp = true);

  double radius() const; 
  OdGePoint3d centerPoint() const;

  virtual bool isBounded() const;

  OdResult setCenterPoint(const OdGePoint3d& center);
  OdResult setRadius(const double val);

  OdResult set(const OdGePoint3d centerPt, const double rad);

  const OdGeCircArc3d getOdGeCircArc3d() const;
  const OdGeCircArc3d getOriginalOdGeCircArc3d() const;
}; 


class DBCONSTRAINTS_EXPORT OdConstrainedArc: public OdConstrainedCircle 
{ 
public:
  ODRX_DECLARE_MEMBERS(OdConstrainedArc);

  OdConstrainedArc();

  OdGePoint3d startPoint() const;
  OdGePoint3d endPoint() const;
  OdGePoint3d midPoint() const;

  virtual bool isBounded() const;

  OdResult set(const OdGePoint3d centerPt, const double rad, const OdGePoint3d startPt, const OdGePoint3d endPt);

  OdResult setStartPoint(const OdGePoint3d& val);
  OdResult setEndPoint(const OdGePoint3d& val);
};

class DBCONSTRAINTS_EXPORT OdConstrainedEllipse: public OdConstrainedCurve 
{ 
public:
  ODRX_DECLARE_MEMBERS(OdConstrainedEllipse);

  OdConstrainedEllipse(bool bCreateImp = true);

  virtual bool isBounded() const;

  OdGeVector3d direction() const; 
  double majorRadius() const; 
  double minorRadius() const;
  OdGePoint3d centerPoint() const;

  OdResult setDirection(const OdGeVector3d& val); 
  OdResult setMajorRadius(const double val); 
  OdResult setMinorRadius(const double val);
  OdResult setCenterPoint(const OdGePoint3d& val);

  OdResult set(const OdGePoint3d centerPt, const OdGeVector3d dir, const double majorRad, const double minorRad);

  const OdGeEllipArc3d getOdGeEllipArc3d() const;
  const OdGeEllipArc3d getOriginalOdGeEllipArc3d() const;
};

class DBCONSTRAINTS_EXPORT OdConstrainedBoundedEllipse : public OdConstrainedEllipse
{ 
public:
  ODRX_DECLARE_MEMBERS(OdConstrainedBoundedEllipse);

  OdConstrainedBoundedEllipse();

  OdGePoint3d startPoint() const;
  OdGePoint3d endPoint() const;

  OdResult setStartPoint(const OdGePoint3d& val);
  OdResult setEndPoint(const OdGePoint3d& val);

  OdResult set(const OdGePoint3d centerPt, const OdGeVector3d dir, const double majorRad, const double minorRad, const OdGePoint3d startPt, const OdGePoint3d endPt);
protected:
  OdGePoint3d m_start;
  OdGePoint3d m_end;
};

class DBCONSTRAINTS_EXPORT OdConstrainedSpline: public OdConstrainedCurve 
{ 
public:
  ODRX_DECLARE_MEMBERS(OdConstrainedSpline);

  OdConstrainedSpline();

  virtual OdResult init(const OdDbObjectId depId, const OdGeNurbCurve3d& spline);
  virtual bool isBounded() const;
  const OdGeNurbCurve3d nurbSpline() const;
  const OdGeNurbCurve3d originalNurbSpline() const;

  int numOfDefinePoints() const;

  OdGePoint3d definePointAt(int index) const;

};

class DBCONSTRAINTS_EXPORT OdConstrainedRigidSet : public OdConstrainedGeometry
{
public:
  ODRX_DECLARE_MEMBERS(OdConstrainedRigidSet);

  OdConstrainedRigidSet();

  int                      numOfConstrainedGeoms() const;
  OdConstrainedGeometry*   getConstrainedGeomAt(int index) const;
  OdGeMatrix3d             transform() const;

  void setTransform(const OdGeMatrix3d &val);
};

#include "TD_PackPop.h"

#endif
