/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////

#ifndef OdExplicitConstraint_INCLUDED_
#define OdExplicitConstraint_INCLUDED_ /* {Secret} */

#include "OdGeomConstraint.h"
#include "Ge/GeVector3d.h"

#include "TD_PackPush.h"

class DBCONSTRAINTS_EXPORT OdExplicitConstraint : public OdGeomConstraint
{
public:
  ODRX_DECLARE_MEMBERS(OdExplicitConstraint);

  OdDbObjectId valueDependencyId() const;
  OdDbObjectId dimDependencyId() const;
  OdResult setValueDependencyId(const OdDbObjectId& valueDependencyId);
  OdResult setDimDependencyId(const OdDbObjectId& dimDependencyId);
  OdResult getMeasuredValue(double& val) const;

  virtual void updateDimDependency(const OdGeMatrix3d& matrToWrd) = 0;
protected:
  OdExplicitConstraint(){};
};

class DBCONSTRAINTS_EXPORT OdDistanceConstraint: public OdExplicitConstraint
{ 
public:
  ODRX_DECLARE_MEMBERS(OdDistanceConstraint);

  enum DirectionType
  {
    kNotDirected         = 0,
    kFixedDirection         ,
    kPerpendicularToLine    ,
    kParallelToLine
  };

  OdDistanceConstraint();
  //OdDistanceConstraint(const OdGeVector3d& direction);
  //OdDistanceConstraint(const OdConstraintGroupNodeId consLineId, DirectionType type = kPerpendicularToLine);
  OdResult setDirection(const OdGeVector3d& direction);
  OdResult setDirection(const OdConstraintGroupNodeId consLineId, DirectionType type = kPerpendicularToLine);
  DirectionType directionType() const;
  OdGeVector3d direction()           const;
  OdConstraintGroupNodeId constrainedLineId() const;

  virtual void updateDimDependency(const OdGeMatrix3d& matrToWrd);
};

class DBCONSTRAINTS_EXPORT OdAngleConstraint: public OdExplicitConstraint 
{ 
public:
  ODRX_DECLARE_MEMBERS(OdAngleConstraint);

  enum SectorType
  {
    kParallelAntiClockwise = 0,
    kAntiParallelClockwise    ,
    kParallelClockwise        ,
    kAntiParallelAntiClockwise    
  };

  OdAngleConstraint();
  //OdAngleConstraint(OdAngleConstraint::SectorType type);

  SectorType sectorType() const;
  OdResult setSectorType(OdAngleConstraint::SectorType type);

  static void setAngleMultiplier(double multiplier);
  static double angleMultiplier();

  virtual void updateDimDependency(const OdGeMatrix3d& matrToWrd);
};

class DBCONSTRAINTS_EXPORT Od3PointAngleConstraint: public OdAngleConstraint 
{ 
public:
  ODRX_DECLARE_MEMBERS(Od3PointAngleConstraint);

  Od3PointAngleConstraint();
  //Od3PointAngleConstraint(OdAngleConstraint::SectorType type);

  virtual void updateDimDependency(const OdGeMatrix3d& matrToWrd);
};

class DBCONSTRAINTS_EXPORT OdRadiusDiameterConstraint : public OdExplicitConstraint
{
public:
  ODRX_DECLARE_MEMBERS(OdRadiusDiameterConstraint);

  enum RadiusDiameterConstrType
  { 
    kCircleRadius   = 0,
    kCircleDiameter    ,
    kMinorRadius       ,
    kMajorRadius
  }; 

  OdRadiusDiameterConstraint();

  RadiusDiameterConstrType constrType() const;
  OdResult setConstrType(RadiusDiameterConstrType type);

  virtual void updateDimDependency(const OdGeMatrix3d& matrToWrd);
};

#include "TD_PackPop.h"
#endif
